% MATGEN 5.0 (Matrix Generator)
% 
% Generates matrices for the multimarked bidding problem.
%
% Written by Anders Lund Eriksrud March 2013
%
% 2.0: Imbalance price not stochastic, with stage aggregation 
% 3.0: A matrix divided into 3
% 3.1: Inflow converted into m^3/s (from m^3/s)
% 3.2: Start-up costs made optional
% 4.0: Piecewise linear decision rules implemented
% 4.1: Stochastic inflow made optional
% 5.0: New approach to stage aggregation


K       = K./Exchange_rate;                     % NOK -> EUR
V       = V./100./Exchange_rate;                % ORE -> EUR
F       = 400000 + F./Exchange_rate;            % NOK -> EUR
for reservoir = 1:2
    R(reservoir,:)  = R(reservoir,:).*R_max(reservoir);
end                                     % Percent -> m^3



%%%%%%%%%%%%%%%%%%%%%%%%%% CREATE A-MATRICIES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
A1       = cell(S,S);
for s = 1:S
    for tau = 1:s
        A1{s,tau} = zeros(m1_s(s),n_s(tau));
    end
end

A1_1 = zeros(m1_s(3),n_s(3));          % Reservoir balance
A1_1(:,1:2) = [-1 0;1 -1; 1 0;-1 1]; 
A1_1(:,6:8) = [-1 0 0;1 -1 -1; 1 0 0;-1 1 1];
A1_2 = agg*A1_1(:,1:end-2);            % Reservoir balance aggregated hours

for s = 3:S
    for tau = 3:min(s,26)
        A1{s,tau} = A1_1;              % Without stage aggregation
    end
    for tau = 27:s
        A1{s,tau} = A1_2;              % With stage aggregation
    end
end
A1{S,S} = [A1{S,S} zeros(m1_s(s-1),1)];     % Add column for v variable

% Add water value cuts to A1:
WV_mat = [];
for p = 1:n_wv
    WV_mat = [WV_mat; [(V(1,p)-V(2,p)) V(2,p) 0 0 0 ...
        (V(1,p)-V(2,p)) V(2,p) V(2,p) zeros(1,6)]];
end
WV_mat1 = [agg*WV_mat ones(n_wv,1)];        % Water value constraints s=S
WV_mat2 = agg*WV_mat;                       % Water value constraints s>26
WV_mat3 = [WV_mat zeros(n_wv,2)];           % Water value constraints s<=26
A1{S,S} = [A1{S,S};WV_mat1];
for tau = 3:26
    A1{S,tau} = [A1{S,tau};WV_mat3];        % Not aggregated
end
for tau = 27:S-1
    A1{S,tau} = [A1{S,tau};WV_mat2];        % Aggregated
end

% Start-up constraints (A2):
A2              = cell(S,2);
A2_1            = zeros(m2_s(4), n_s(4));
A2_2            = zeros(m2_s(4), n_s(4));
if startup(4)
    A2_1(:,9:11)    = -diag(K);             % u last hour
    A2_2(:,9:11)    = diag(K);             	% u this hour
    A2_2(:,3:5)     = -eye(3);           	% o (start-up costs)
end
for s = 4:26
    if startup(s)
        A2{s,1} = A2_1;
        A2{s,2} = A2_2;
    end
end
if startup(27)
    A2{27,1} = A2_1;
    A2{27,2} = A2_2(:,1:end-2);
end
for s = 28:S
    if startup(s)
        A2{s,1} = A2_1(:,1:end-2);
        A2{s,2} = A2_2(:,1:end-2);
    end
end
if startup(S)
    A2{S,2} = [A2{S,2} zeros(m2_s(S),1)];
end

A3_1su  = zeros(m3_s(4), n_s(3));       % Start-up matrices

% P-Q-curves:
A3_1su(1:3,6) = -E(1,:)';
A3_1su(1:3,12) = ones(3,1);
A3_1su(4:6,7) = -E(2,:)';
A3_1su(4:6,13) = ones(3,1);
A3_1su(7:9,8) = -E(3,:)';
A3_1su(7:9,14) = ones(3,1);

A3_1su(10:15,12:14) = [eye(3);-eye(3)]; % Production bounds
A3_1su(16:17,1:2) = -eye(2);            % Nonnegative spill
A3_1su(18:20,6:8) = -eye(3);            % Nonnegative discharges
A3_1su(21:22,end-1:end) = -eye(2);      % Nonnegative imbalances

A3_1wsu = A3_1su(1:22,:);               % No Start-up matricies

A3_1su(10:15,9:11) = [-diag(W_max); diag(W_min)];    
                                        % Production bounds (dep on u)
A3_1su(23:25,3:5) = -eye(3);            % Nonnegative start-up costs
A3_1su(26:31, 9:11) = [eye(3);-eye(3)]; % Bounds on u-variables

A3_2su      = A3_1su([1:20 23:31],1:end-2); 
                                        % After first day (no imbalances)
A3_2wsu     = A3_1wsu(1:end-2,1:end-2); % -''- without startup costs 


A3 = cell(S,1);
for s = 1:S
    A3{s} = zeros(m3_s(s), n_s(s));
end

if startup(3)
    A3{3} = [A3_1su;A2_2];        % Add start-up constraints for first hour
else
    A3{3} = A3_1wsu;
end
for s = 4:26
    if startup(s)
        A3{s} = A3_1su;
    else
        A3{s} = A3_1wsu;
    end
end
for s = 27:S
    if startup(s)
        A3{s} = A3_2su;
    else
        A3{s} = A3_2wsu;
    end
end
A3{S} = [A3{S} zeros(m3_s(S),1)];       % Add column for v variable

A_eq = cell(26,26);                     % only needed for first day
m_eq = [0 0 ones(1,24)]';
for s = 1:26
    for tau = 1:s
        A_eq{s,tau} = zeros(m_eq(s), n_s(tau));
    end
end
for s = 3:26
    A_eq{s,2} = zeros(1,24);
    A_eq{s,2}(s-2) = 1;
    A_eq{s,s} = [zeros(1,11) -1 -1 -1 1 -1];
end

    
%%%%%%%%%%%%%%%%%%%%%%%%%% CREATE B-MATRICIES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
B1 = cell(S,1);
for s = 1:S
    B1{s} = zeros(m1_s(s), ks(s));
end

B1_1 = zeros(m1_s(3),ks(2));
B1_1(:,1) = [(R_max(1)-R_0(1));(R_max(2)-R_0(2));(-R_min(1)+R_0(1));...
             (-R_min(2)+R_0(2))];           % Reservoir balances

mean_infl = 3600*[mean(inflow(:,:,1))' mean(inflow(:,:,2))'];
                % scale up to m^3/h (from m^3/s)      
                
if stoc_imb
    error('Stochastic imbalance price not implemented');
end

if stoc_infl    % Stochastic inflow
    mat = 3600*[-1 0;0 -1;1 0;0 1];         % Inflows (scale up to m^3/h]
    B1_1 = [B1_1 mat];                      % s = 3 must be new stage
    B1{3} = B1_1;
    B1{3}(:,1) = B1{3}(:,1) + [-mean_infl(1,1);-mean_infl(1,2);...
                                mean_infl(1,1);mean_infl(1,2)];
else % Determinstic inflow
    B1{3} = B1_1;
    B1{3}(:,1) = B1{3}(:,1) + [-mean_infl(1,:)'; mean_infl(1,:)'];    
end
                            
for s = 4:26
    if stoc_infl
        if sum(aggs==s) == 1
            B1{s} = [B1{s-1} mat];          % add new xi if new stage
        else
            B1{s} = B1{s-1};
            B1{s}(:,end-1:end) = B1{s}(:,end-1:end) + mat;  
                                            % add one stochastic error for
                                            % each of the hours in the
                                            % macroperiod
        end
    else % Deterministic Inflow
        B1{s} = B1{s-1}; 
    end
        B1{s}(:,1) = B1{s}(:,1) + [-mean_infl(s-2,:)'; mean_infl(s-2,:)'];
                                            % add mean inflow hour t = s-2                                           
end
if stoc_infl
    mat2 = agg*[mat zeros(m1_s(3),1)];      % After first day, aggregate
end
for s = 27:S
    if stoc_infl
        if sum(aggs==s) == 1
          	B1{s} = [B1{s-1} mat2];
        else
            B1{s} = B1{s-1};
            B1{s}(:,end-2:end) = B1{s}(:,end-2:end) + mat2;  
        end
    else % Deterministic inflow
        B1{s} = [B1{s-1} zeros(m1_s(s-1), k_s(s))];  % (s-1) because m(S)>4
    end
    t_start = 25 + agg*(s-27);
    t_stop = 24 + agg*(s-26);
    if agg >= 2
        B1{s}(:,1) = B1{s}(:,1) + [-sum(mean_infl(t_start:t_stop,:))';...
                                    sum(mean_infl(t_start:t_stop,:))'];
    else % agg == 1
        B1{s}(:,1) = B1{s}(:,1) + [-mean_infl(t_start:t_stop,:)';...
                                    mean_infl(t_start:t_stop,:)'];
    end
end
    
   
% Water value constraints
B_WV = [];
if stoc_imb
    error('Stochastic imbalance price not implemented');
end
for p = 1:n_wv
    if stoc_infl
        mat_1 = [];
        for s = 3:26
            if sum(aggs==s) == 1
                mat_1 = [mat_1 3600*[V(1,p) V(2,p)]];
            else
                mat_1(end-1:end) = mat_1(end-1:end) + 3600*[V(1,p) V(2,p)];
            end                               % Add inflow aggregated hours
        end
        mat_2 = [];
        for s = 27:S
            if sum(aggs==s) == 1
                mat_2 = [mat_2 3600*[agg*V(1,p) agg*V(2,p) 0]];
            else
                mat_2(end-2:end-1) = mat_2(end-2:end-1)...
                                     + 3600*[agg*V(1,p) agg*V(2,p)];
            end
        end
        add_to_B_WV = [(F(p)-V(1,p)*R(1,p)+V(1,p)*R_0(1)-V(2,p)*R(2,p)+...
                        V(2,p)*R_0(2)+V(1,p)*sum(mean_infl(:,1))+...
                        V(2,p)*sum(mean_infl(:,2))) zeros(1,k_s(2)) mat_1...
                        mat_2];
    else % Deterministic inflow
        add_to_B_WV = [(F(p)-V(1,p)*R(1,p)+V(1,p)*R_0(1)-V(2,p)*R(2,p)+...
                        V(2,p)*R_0(2)+V(1,p)*sum(mean_infl(:,1))+...
                        V(2,p)*sum(mean_infl(:,2))) zeros(1,k-1)];
    end   
    B_WV = [B_WV; add_to_B_WV];
end
B1{S} = [B1{S}; B_WV];

% B3:
B3 = cell(S,1);
for s = 1:S
    B3{s} = zeros(m3_s(s), ks(s));
    if s >= 3
        B3{s}(1:9,1) = [E_hat(1,:) E_hat(2,:) E_hat(3,:)]'; % P-Q-curve
        if startup(s)
            if s == 3      % in t = 3 we have more constraints (lower)
                B3{s}(end-8:end-6, 1) = 1;                  % Bound on u
            else
                B3{s}(end-5:end-3, 1) = 1;                  % Bound on u
            end
        else
            B3{s}(10:15,1) = [W_max;zeros(3,1)];            % Bounds on w
        end
    end
end
if startup(3)
    B3{3}(end-2:end,1) = K.*u_0;
end






%%%%%%%%%%%%%%%%%%%%%%%%%% CREATE C-MATRICIES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C = cell(S,1);
for s = 1:S
    C{s} = zeros(n_s(s), ks(s));
end

spot_mean = mean(spot)';

if piecewise && ~prins_comp
    C{2}(1,1) = -rho_low(1);
    
    % t = 1:
    C{3}(3:5,1) = 1;        % Start up costs;
    % Imbalance prices:
    C{3}(end-1,1) = imb;
    C{3}(end,1) = imb;
    C{3}(end-1,2:r_i(1)+1) = -1;
    C{3}(end,2:r_i(1)+1) = 1;
    for t = 2:24
        C{2}(t,1) = -rho_low(t);
        C{2}(t, sum(r_i(1:t-1))+2:sum(r_i(1:t))+1) = -1;
        C{t+2}(3:5,1) = 1;      % Start-up costs
        % Imbalance Price:
        C{t+2}(end-1,1) = imb;
        C{t+2}(end,1) = imb;
        C{t+2}(end-1,sum(r_i(1:t-1))+2:sum(r_i(1:t))+1) = -1;
        C{t+2}(end,sum(r_i(1:t-1))+2:sum(r_i(1:t))+1) = 1;
    end
elseif ~piecewise 
    C{2} = [zeros(n_s(2),1) -eye(n_s(2))];
    C{2}(:,1) = -spot_mean';
    for s = 3:26    % Imbalance prices:
        C{s}(3:5,1) = 1;        % Start-up costs
        % Imbalance Price:
        C{s}(end-1,s-1) = -1; 
        C{s}(end,s-1) = 1;  
        C{s}(end-1,1) = imb; 
        C{s}(end,1) = imb; 
    end   
elseif prins_comp
    C{2} = zeros(n_s(2),ksL(2));
    C{2}(1,2:r_i(1)+1) = -1;
    for t = 2:24
        C{2}(t, sum(r_i(1:t-1))+2:sum(r_i(1:t))+1) = -1;
    end
    
    for s = 3:26    % Imbalance prices:
        C{s} = zeros(n_s(s), 25);       % 25 = original k^s
        C{s}(3:5,1) = 1;        % Start-up costs
        % Imbalance Price:
        C{s}(end-1,s-1) = -1; 
        C{s}(end,s-1) = 1;  
        C{s}(end-1,1) = imb; 
        C{s}(end,1) = imb; 
    end
end
    


for s = 27:S-1  
    C{s}(3:5,1) = 1;        % Start-up costs
    % Spot income:
    t_start = 25 + agg*(s-27);
    t_end = 24 + agg*(s-26);
    C{s}(end-2:end,1) = -sum(spot_mean(t_start:t_end));
    C{s}(end-2:end,end) = -agg;
end
% Last hour (include water value variable)
C{S}(3:5,1) = 1;
C{S}(end-3:end-1,1) = -sum(spot_mean(24*days-agg+1:24*days));
C{S}(end-3:end-1,end) = -agg;
C{S}(end,1) = -1;





