% SCENGEN 4.1 (Scenario Generator)
%
% Generates scenarios to LDR multimarked model
%
% Written by Anders Lund Eriksrud, March 2013
%
% 2.0: Removed imbalance price as stochastic variable
% 3.0: Implemented bounds on ARMA-residuals for spot price and total inflow
%      (optional)
% 3.1: Inflow converted to m^3/s (from m^3/h)
% 4.0: Piecewise LDR implemented
% 4.1: Inflow made optional (does not affect SCENGEN, only SCENMAT)

%-----INPUT----------------------------------------------------------------

start_day           = 4;        % 1 = mon, 2 = tue, ..., 7 = sun (tomorrow)
                    % 4.10 = thursday
training_period     = 8;        % Weeks
    

%--------------------------------------------------------------------------

day_start   = 8;                % hour in which day-period begins (7am)

                
% ---------------------- Generate inflow scenarios ------------------------
infl(:,:,1)     = load('Input/Inflow1.txt')./3600;
infl(:,:,2)     = load('Input/Inflow2.txt')./3600;
n_inf           = 50;   % # of inflow scenarios
inflow          = zeros(days*24, n_scen, 2);

for s = 1:n_scen
    scen_pick   = floor(rand*n_inf)+1; 	% picks out ruling inflow scenario
    for reservoir = 1:2                 % loop thru all reservoirs
        inflow(:,s,reservoir) = infl(:,scen_pick,reservoir); 
    end
end



%------------------------ Generate spot scenarios -------------------------
data        = load('Input/Spot1004.txt');       % load forecast error data
vars        = spotvar(data, 8);                 % Estimates the variance 
                            % for each hour for the spot forecast error, 
                            % split for weekdays (1) and weekends (2)
   
spot_prog   = load('Input/spotProg.txt');       % loads spot prognosis 
                                                % (entire period)  [NOK]
spot_prog   = spot_prog/exchange_rate;          % [EUR]

parameters  = load('R/progpar.txt');            % loads spot parameters
                                                % row 1 = AR1, row 2 = AR2,
                                                % row 3 = MA1, row 4 = 
                                                % SAR1, row 5 = mean.
                                                
init_prog   = load('R/progdata.txt');           % Initial data
prog_err    = load('R/progres.txt');            % Residuals prognosis error
n_pe        = length(prog_err);

spot        = zeros(days*24, n_scen);         % Spot prices
sigma       = zeros(24, n_scen, 2);             % Imbalance prices (1 = +)

for s = 1:n_scen
    curr_prog   = init_prog(2:end);
    t = 1;      % error hour 1 is drawn from a normal distribution around 0
    err         = normrnd(0,vars(1,1)^.5);
    spot(t,s)   = spot_prog(t) + err;
    curr_res    = err - parameters(5) - parameters(1)*curr_prog(24) ...
        - parameters(2)*curr_prog(23) - parameters(3)*prog_err(n_pe) ...
        - parameters(4)*curr_prog(1);    % calculate first residual
    curr_prog(1:23) = curr_prog(2:24);
    curr_prog(24)   = err;
    sigma(t,s,1)    = spot(t,s) - 6;
    sigma(t,s,2)    = spot(t,s) + 6;
    for t = 2:days*24
        err       = prog_err(floor(n_pe*rand)+1);
        estimate    = parameters(1)*curr_prog(24)...        % AR1
                      + parameters(2)*curr_prog(23)...      % AR2
                      + parameters(3)*curr_res...           % MA1
                      + parameters(4)*curr_prog(1);         % SAR1
        spot(t,s)   = spot_prog(t) + estimate + err + parameters(5);
        % Update curr_prog:
        curr_prog(1:23) = curr_prog(2:24);
        curr_prog(24)   = estimate + err;
        % Update curr_res:
        curr_res        = err;
        if(t<=24)       % Define imbalance price (bidding day)
            sigma(t,s,1)    = spot(t,s) - 7 + 2*rand;
            sigma(t,s,2)    = spot(t,s) + 5 + 2*rand;
        end
    end
    
end



k_all = ks(2) + k_s(3)*24 + k_s(27)*(days-1)*24;

xi  = [ones(n_scen,1) spot(1:24,:)'  zeros(n_scen, k_all-ks(2))]; 

for stage = 3:26
    i = 25 + k_s(3)*(stage - 3);
    for j = 1:2
        xi(:,i+j) = inflow(stage-2,:,j)';
    end
    for j = 1:2
        xi(:,i+j+2) = sigma(stage-2,:,j)';
    end
end
for stage = 27:(2+24*days)
    i = ks(26) + k_s(27)*(stage - 27);
    for j = 1:2
        xi(:,i+j) = inflow(stage-2,:,j)';
    end
    xi(:,i+3) = spot(stage-2,:)';
end


fname = 'xi_tot.txt';
fid = fopen(fname,'w');
dlmwrite(fname, xi, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
fclose(fid);




