% SCENMAT 2.0 (Scenario Matrix Generator)
%
% Constructs the matricies M, W and h, based on xi_tot.txt (generated from
% SCENGEN and options made in MATWRITER
%
% 1.0: SCENGEN splitted into SCENGEN and SCENMAT, piecewise LDR implemented
% 1.1: Stochastic inflow made optional
% 2.0: New approach to stage aggregation

clear inflow;
clear spot;
clear xi;
xi_old = load('xi_tot.txt');    % Load scenarios



%----------------------- Extract spot and inflow --------------------------

spot = xi_old(:,2:25);          % spot first day
inflow = zeros(n_scen, 24*days, 2);
for t = 1:24
    i = ks(2) + 2*(t-1);        % 2 entered manually to fit xi_old
    for j = 1:2
        inflow(:,t,j) = xi_old(:,i+j);
    end
end
for t = 25:24*days
    i = ks(2) + 2*24 + 3*(t-25);    % adjusted manually to fit xi_old
    for j = 1:2
        inflow(:,t,j) = xi_old(:,i+j);
    end
end
for t = 25:24*days
    i = ks(2) + 2*24 + 3*(t-24);    % adjusted manually to fit xi_old
    spot(:,t) = xi_old(:,i);
end


%---------------------- Aggregate hours (using agg) -----------------------
xi_old  = [ones(n_scen,1) spot(:,1:24)  zeros(n_scen, k-ks(2))]; 
if stoc_infl
    for stage = 3:26
        i = 25 + k_s(3)*(stage - 3);
        for j = 1:2
            xi_old(:,i+j) = inflow(:,stage-2,j);
        end
    end
else
    i = 24;
    j = 1;
end

start_i = i + j;
for stage = 27:S
    t_s = 25 + agg*(stage-27);                      % start time period
    i = start_i + k_s(27)*(stage - 27);
    
    if stoc_infl
        for res = 1:2       % Reservoir
            av = zeros(n_scen,1);
            for a = 1:agg
                av = av + inflow(:,t_s+a-1,res)./agg;    % average inflow
            end
            xi_old(:,i+res) = av;
        end
    end
    
    av = zeros(n_scen,1);
    for a = 1:agg
        av = av + spot(:,t_s+a-1)./agg;              % average spot price
    end
    xi_old(:,i+k_s(27)) = av;
end


%------------------- Stage aggregation (using aggs) -----------------------
nu_or = mean(xi_old)';
M = length(aggs);               % Number of macroperiods
ks_m(1:2) = ks(1:2);
for m = 3:M
    ks_m(m) = ks_m(m-1) + k_s(aggs(m));
end
k_m = ks_m(end);

xi = zeros(n_scen, k_m);          % New xi matrix

xi(:,1:ks(2)) = xi_old(:,1:ks(2));    % xi_1 (ones) and xi_2 (spot prices)

if stoc_imb
    error('Stochastic imbalance prices not implemented');
end

if ~piecewise   % Use xi as error from expected value
    spot_mean = mean(xi(:,2:25));
    for i = 2:25
        xi(:,i) = xi(:,i) - spot_mean(i-1);
    end
end

for m = 3:M
    av = 0;
    if m == M
        for s = aggs(m):S
            av = av + xi_old(:,ks(s-1)+1:ks(s)) - repmat(nu_or(ks(s-1)+1:ks(s))', [n_scen,1]);
        end
        av = av/(S+1-aggs(M));
    else
        for s = aggs(m):aggs(m+1)-1
            av = av  + xi_old(:,ks(s-1)+1:ks(s)) - repmat(nu_or(ks(s-1)+1:ks(s))', [n_scen,1]);
        end
        av = av/(aggs(m+1)-aggs(m));        % average deviations from nu_or
    end
    xi(:,ks_m(m-1)+1:ks_m(m)) = av;
end
k_s(1) = ks_m(1);
for s = 2:S
    if sum(aggs == s) == 1
        m = find(aggs == s);
        k_s(s) = ks_m(m) - ks_m(m-1);
    else
        k_s(s) = 0;
    end
    ks(s) = ks(s-1) + k_s(s);
end
k = ks(end);


%----------------------- Lifting (piecewise LDR) --------------------------
if piecewise && ~prins_comp
    xi_min = min(xi(:,2:25));       % min spot prices
    xi_max = max(xi(:,2:25));       % max spot prices
    r_i = ones(24,1);
    ppoints = cell(24,1);
    for i = 1:24
      	ppoints{i} = pricepoints; 
     	ok = false;
     	while ~ok && length(ppoints{i}) >= 1
        	if xi_min(i) >= ppoints{i}(1)  % rho_min must be < first price point
             	ppoints{i}(1) = [];
         	elseif xi_max(i) <= ppoints{i}(end) % rho_max must be > last price point
             	ppoints{i}(end) = [];
           	else
              	ok = true;
        	end
        end
        r_i(i) = length(ppoints{i})+1;
        if r_i(i) == 1
            error('Your pricepoints are badly defined');
        end
    end

    
    ksL(1) = ks(1);                 % Lifted k^s
    ksL(2) = ksL(1) + sum(r_i);
    for s = 3:S
        ksL(s) = ksL(s-1) + k_s(s);
    end
    kL = ksL(end);
    

    
    
    xiL = zeros(n_scen, kL);        % Lifted uncertain parameters
    xiL(:,1) = xi(:,1);
    i_L = 1;
    for i = 2:25                    % r_i > 1 (tested above)
        i_L = i_L + 1;
        xiL(:,i_L) = min(xi(:,i), ppoints{i-1}(1)) - xi_min(i-1);
        for j = 2:r_i(i-1)-1
            i_L = i_L + 1;
            xiL(:,i_L) = max(min(xi(:,i),ppoints{i-1}(j))-ppoints{i-1}(j-1),0);
        end
        i_L = i_L + 1;
        xiL(:,i_L) = max(xi(:,i)-ppoints{i-1}(r_i(i-1)-1),0);
    end
    xiL(:,ksL(2)+1:end) = xi(:,ks(2)+1:end);
    
    
    % Convert min and max for lifted parameters (relative to xi_min):
    rho_low = xi_min;
    xi_max = xi_max - xi_min;
    xi_min = zeros(24,1);
    for t = 1:24
        ppoints{t} = ppoints{t} - rho_low(t);
    end
    
    if length(find((min(xiL)-max(xiL))==0)) >= 2   
        % Test whether some lifted parameters are deterministic
        error('At least one pricepoint is redundant for one hour');
    end
        
    xi      = xiL;
    ks      = ksL;
    k       = kL;
    k_s(2)  = sum(r_i);
    
    W = [[1;-1] zeros(2,k-1)];  % bounds on xi_1 := 1
    h = [1;-1];
    
    % Generate W_i matrices:
    for i = 2:25
        t = i - 1;
        W_i = zeros(r_i(t)+1, r_i(t));      % first row created separately
        W_i1 = zeros(r_i(t)+1,1);           % first row
        W_i1(1:2) = [ppoints{t}(1)/(ppoints{t}(1)-xi_min(t));...
                     -xi_min(t)/(ppoints{t}(1)-xi_min(t))];
        W_i(1:2,1) = [-1/(ppoints{t}(1)-xi_min(t));...
                      1/(ppoints{t}(1)-xi_min(t))];
        for j = 2:r_i(t)-1
            W_i(j:j+1,j) = [-1/(ppoints{t}(j)-ppoints{t}(j-1));...
                            1/(ppoints{t}(j)-ppoints{t}(j-1))];
        end
        W_i(end-1:end, end) = [-1/(xi_max(t)-ppoints{t}(end));...
                                1/(xi_max(t)-ppoints{t}(end))];
        
        W_add = [W_i1 zeros(r_i(t)+1,k-1)];
        if t == 1
            W_add(:, 2:r_i(t)+1) = W_i;
        else
            W_add(:, (sum(r_i(1:t-1))+2):(sum(r_i(1:t))+1)) = W_i;
        end
        W = [W; W_add];
        h = [h;zeros(r_i(t)+1,1)];
    end
    
    % Upper and lower bounds on xi for s > 2:
    W_add = zeros((k-ks(2))*2,k);
    W_add(:,ks(2)+1:end) = [eye(k-ks(2));-eye(k-ks(2))];
    W = [W;W_add];
    h = [h;min(xi(:,ks(2)+1:end))'; -max(xi(:,ks(2)+1:end))'];
    
%--------------------------------------------------------------------------
    
elseif ~piecewise        % Not piecewise LDR
    % Calculate W and h
    Wmin = eye(k);  
    Wmax = -eye(k);
    h_min = min(xi)';
    h_max = -max(xi)';
    W = [Wmin;Wmax];
    h = [h_min;h_max];
end

if prins_comp
    run('prinsred');
end


Mtgen;  % run M_t generator

% Calculate M:
Mom = zeros(k,k);
for i = 1:k
    for j = i:k
        temp = mean(xi(:,i).*xi(:,j));
        Mom(i,j) = temp;
        if i ~= j
            Mom(j,i) = temp;
        end
    end
end

% Test feasibility
minTest = inf;
for s = 1:n_scen
    if  min(W*xi(s,:)' - h) < minTest
        [minTest, row_err] = min(W*xi(s,:)' - h);
    end
end
if minTest < -0.0000001
    errormsg = ['The system W*xi >= h is not feasible. ' ...
                'Error located in row ' num2str(row_err) ' of W'];
    error(errormsg);
end

% Test boundedness
x = mean(xi)';
minT = min(W*x-h);
for i = 1:length(x)
    x_i = x;
    x_i(i) = x_i(i) + 10^12;
    mint = min(W*x_i-h);
    if mint >= -eps
        error('The polehydron W*xi >= h is not bounded');
    end
    x_i = x;
    x_i(i) = x_i(i) - 10^12;
    mint = min(W*x_i-h);
    if mint >= -eps
        error('The polehydron W*xi >= h is not bounded');
    end
end
