% MATWRITER 4.0 (Matrix Writer)
%
% Writes out input file for Xpress LDR solver for the multimarked bidding
% problem. The script creates a new folder in which the is put. The folder 
% name is Output/Output [date time].
%
% Please fill in the INPUT and OPTION tabs in the code.
%
% OUTPUT FILES:
% DETdata.txt:      Input file to the deterministic solver 
%                   'exp_determinstic.mos'.
% DUALdata.txt:     Input file to the dual solver 'LDR_dual.mos'.
% EXPdata.txt:      Input file to the deterministic solver 
%                   'matrix_det.mos'.
% intData.m:        Input file to the interpreter 'interpreter.m'.
% LDRdata.txt:      Input file to the LDR solver 'LDR_primal.mos'.
% PriceData.txt:    Input to scenario based solver
%                   'scenarioFormulation.mos'.
% readme.txt:       A text file that displays the main input and options 
%                   for the relevant data files.
% scenarioData.txt: Input file to SCENRED (spot scenarios).
% xi.txt:           Input file (scenarios) to the intepreter 
%                   'interpreter.m'.
% 
%
% SUBSCRIPTS:
% Scengen/scengen.m:    Performs the scenario generation.
% Scengen/scenmat.m:    Calaculates the M, W and h matrices (support
%                       description), based on the inputs and options.
% Matgen/matgen.m:      Generates the matrices A, B and C (manually) for   
%                       the spot bidding problem.
%
%
% Written by Anders Lund Eriksrud March-April 2013.
%
% UPDATES:
% 2.0: writes out B instead of P and G.
% 3.0: divides A into 3 submatricies.
% 3.1: inflow converted to m^3/s (from m^3/h), start-up made optional.
% 3.2: Piecewise LDR implemented, new xi made optional, stochastic inflow
%      made optional.
% 4.0: New approach to stage aggreagation.
% 4.1: New datafiles
% 4.2: Principal component reduction implemented

clc;clear;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% INPUT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Number of hours aggregated after first day
agg         = 1;       

% Stage aggregation, vector of new stages, must contain = [1 2 3 27]
aggs        = [1 2 3 27];  

% Number of days in planning horizon
days        = 4;    

% Penalty [EUR] for imbalance
imb         = 6;        

% Number of hours with start-up costs (aggregated)
start_h     = 0;        

% Price points spot market bidding curve (OBS: switch piecewise!)
pricepoints = []';

% Number of principal components:
n_pr = 4;

% Number of scenarios in the scenario generation
n_scen      = 3000;     

% Exchange rate EUR/NOK last known date
exchange_rate = 7.3827; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% OPTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Use piecewise LDR?
piecewise       = true;     

% Perform new scenario generation?
new_xi          = false;        

% Use stochastic inflow?
stoc_infl       = false;    

% Use stochastic imbalance?
stoc_imb        = false;        

% Use ARMA restriction in the support description for the spot price? (not
% compatible with piecewise LDR)
include_ARMA    = false;

% Use average inflow restriction in the support description? (not
% compatible with piecewise LDR)
include_avRes   = false;

% Use Principal component reduction
prins_comp      = true;

% Printouts:
LDRdata         = true;
DETdata         = false;
EXPdata         = false;
XIdata          = true;
DUALdata        = false;
scenarioData    = false;
DualOrg         = false;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pricepoints = sort(pricepoints);

if(floor(24/agg) ~= 24/agg && agg <= 24)
    error('Your aggregation number ("agg") is not feasible');
elseif(agg >= 25)
    warning('High aggregation!!');
end
if sum(aggs==1) == 0
    error('Stage 1 must be contained in aggs');
elseif sum(aggs==2) == 0
    error('Stage 2 must be contained in aggs');
elseif sum(aggs==3) == 0
    error('Stage 3 must be contained in aggs');
elseif sum(aggs==27) == 0
    error('Stage 27 must be contained in aggs');
elseif prins_comp && ~piecewise
    error('PCA reduction can only be used with piecewise LDR');
end

run('Z:\Matlab\Matgen\Input\parameters.m');     % Load parameters
run('Z:\Matlab\Matgen\Input\watervalues18.m');  % Load Water value data
n_wv    = size(R,2);                    % Number of water value cuts

S       = 2 + 24 + 24*(days-1)/agg;     % Number of stages 
if aggs(end) > S
    warning(['Elements in "aggs" exceeded the number of time stages, ',...
        'aggs(end) was set to S']);
    overS = sum(aggs > S);
    if aggs(end-overS) == S && overS >= 2
        aggs(end-overS+1:end) = [];
    elseif overS >= 2
        aggs(end-overS+1) = S;
        aggs(end-overS+2:end) = [];
    else   % overS == 1
        aggs(end) = S;
    end
end
    
startup = zeros(S,1);                   % Create vector (flag) of startups
if start_h >= S-2
    start_h = S-2;
end
startup(3:(start_h+2)) = 1;

if stoc_infl
    k_s     = [1 24 2*ones(1,24) 3*ones(1,S-26)]';      
             	% Number of uncertain parameters in each stage
else
    k_s     = [1 24 zeros(1,24) ones(1,S-26)]';  
end
                
n_s     = [0 24 16*ones(1,24) 14*ones(1,S-27) 15]';
                % Number of variables in each stage
m1_s    = [0 0 4*ones(1,S-3) 4+n_wv]';
                % Number of constraints of type 1 (sum_s=1^t)A_ts*x_s
m2_s    = [0 0 0 3*startup(4:end)']';
                % Number of constraints of type 2 (sum_s=(t-1)^t)A_ts*x_s
m3_s    = [0 0 22*ones(1,24) 20*ones(1,S-26)]';
                % Number of constraints of type 3 (A_t*x_t)
m3_s    = m3_s + 9*startup;
if startup(3) == 1
    m3_s(3) = m3_s(3) + 3;          % Add initial start-up constraint
end

ks      = k_s(1);                   
for s = 2:S
    ks(s) = ks(s-1) + k_s(s);   % = k^s
end
k       = ks(end);

if new_xi
    run('Scengen\scengen.m');       % scenario generator
end
run('Scengen\scenmat.m');           % generates W,h,M
run('Matgen\matgen.m');             % generates A,B,C matricies

nu      = Mom(:,1);                 % vector of expected values
l       = length(h);

% Create P_s:
P = cell(S,1);
P_length = 0;
for s = 1:S
    temp = eye(k);
    P{s} = temp(1:ks(s),:);
    P_length = P_length + size(P{s},1)*size(P{s},2);
end

if ~prins_comp
    % Calculate D_s:
    D = cell(S,1);
    D_length = 0;
    for s = 1:S
        D{s} = P{s}*Mom*P{s}'*C{s}';
        D_length = D_length + size(D{s},1)*size(D{s},2);
    end
end

    
A1_length = 0;
for s = 1:S
    for tau = 1:s
        A1_length = A1_length + size(A1{s,tau},1) * size(A1{s,tau},2);
    end
end

A2_length = 0;
for s = 1:S
    for tau = 1:2
        A2_length = A2_length + size(A2{s,tau},1) * size(A2{s,tau},2);
    end
end

A3_length = 0;
for s = 1:S
    A3_length = A3_length + size(A3{s},1) * size(A3{s},2);
end

A_eq_length = 0;
for s =1:26
    for tau = 1:s
        A_eq_length = A_eq_length + size(A_eq{s,tau},1) * size(A_eq{s,tau},2);
    end
end

B1_length = 0;
for s =1:S
    B1_length = B1_length + size(B1{s},1) * size(B1{s},2);
end

B3_length = 0;
for s =1:S
    B3_length = B3_length + size(B3{s},1) * size(B3{s},2);
end

starts = find(startup == 1);


% Print results to file:
    cl = clock; 
    if cl(4) <= 9 && cl(5) <= 9
        time = ['0', num2str(cl(4)), '0', num2str(cl(5))];
    elseif cl(4) <= 9
        time = ['0', num2str(cl(4)), num2str(cl(5))];
    elseif cl(5) <= 9
        time = [num2str(cl(4)), '0', num2str(cl(5))];
    else
        time = [num2str(cl(4)), num2str(cl(5))];
    end
    dirname2 = ['Data ', date, ' ', time];
    dirname = ['Output/', dirname2];
    mkdir('Output', dirname2);
    cd(dirname);
    
    fname = 'readme.txt';
    fid = fopen(fname,'w');
    dlmwrite(fname, dirname2, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    dlmwrite(fname, ['agg: ' num2str(agg)], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ['imb: ' num2str(imb)], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ['start_h: ' num2str(start_h)], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ['Macroperiods starts in (k = ', num2str(k), '):    ', num2str(aggs)], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, ['------------------ OPTIONS: ------------------'],...
        'delimiter', '', 'newline', 'pc', '-append', 'roffset',  2);
    if stoc_infl
        dlmwrite(fname, 'Stochastic inflow', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    else
        dlmwrite(fname, 'Deterministic inflow', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    end
    if piecewise
        dlmwrite(fname, ['Piecewise linear decision rules used with ',...
            'the following pricepoints:      ', num2str(pricepoints')], ...
            'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    else
        dlmwrite(fname, 'Piecewise linear decision rules not used',...
            'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    end
    fclose(fid);



if LDRdata && ~prins_comp
    
    if piecewise
        % Cummulative r_i's:
        r_i_cum = zeros(25,1);
        r_i_cum(1) = 1;
        for t = 1:24
            r_i_cum(t+1) = r_i_cum(t) + r_i(t);
        end
    end
    
    fname = 'LDRdata.txt';
    fid = fopen(fname,'w');
    dlmwrite(fname, ['! ' dirname2], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'T : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, S, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'l : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, l, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'n : [',  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, n_s,  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'starts : [',  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, starts,  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'kt : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, ks', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'm1 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m1_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'm2 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m2_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'm3 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m3_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'piecewise : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, piecewise, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    if piecewise
    dlmwrite(fname, 'r_i_cum : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, r_i_cum, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    else
        dlmwrite(fname, 'r_i_cum : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
            dlmwrite(fname, zeros(25,1), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    end
    dlmwrite(fname, 'A1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A2_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A2_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    
    dlmwrite(fname, 'B1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(B1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'B3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(B3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'D_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(D_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A_eq_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A_eq_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);


    dlmwrite(fname, 'D_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, D{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            for tau = 1:s
                dlmwrite(fname, A1{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A2_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            for tau = 1:2
                dlmwrite(fname, A2{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, A3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A_eq_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:26
            for tau = 1:s
                dlmwrite(fname, A_eq{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'W : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, W, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'B1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, B1{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'B3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, B3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'h : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, h, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'nu : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, nu, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    fclose(fid);
end

if DETdata
    C_length = 0;
    for s =1:S
        C_length = C_length + size(C{s},1) * size(C{s},2);
    end
    
    
    fname = 'DETdata.txt';
    fid = fopen(fname,'w');
    dlmwrite(fname, ['! ' dirname2], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

       
    dlmwrite(fname, 'T : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, S, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'n : [',  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, n_s,  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'kt : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, ks', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'm1 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m1_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'm2 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m2_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'm3 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m3_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A2_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A2_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'starts : [',  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, starts,  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'B1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(B1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'B3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(B3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A_eq_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A_eq_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);


    dlmwrite(fname, 'C_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(C_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    

    dlmwrite(fname, 'A_eq_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:26
            for tau = 1:s
                dlmwrite(fname, A_eq{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'C_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, C{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'nu : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, nu, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    
    
    dlmwrite(fname, 'A1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            for tau = 1:s
                dlmwrite(fname, A1{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A2_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            for tau = 1:2
                dlmwrite(fname, A2{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, A3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    

    
    dlmwrite(fname, 'B1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, B1{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'B3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, B3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    
    fclose(fid);
end

    infl_out(1,:) = 3600*mean(inflow(:,:,1));
    infl_out(2,:) = 3600*mean(inflow(:,:,2));
if EXPdata


    spot_out = mean(spot);

    fname = 'ExpData.txt';
    fid = fopen(fname,'w');
    dlmwrite(fname, ['! ' dirname2], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);


    dlmwrite(fname, 'Inflow_OR : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, infl_out, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'Rho_1_OR : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, spot_out, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'agg : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, agg, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'Im_addition : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, imb, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'Start_h : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, start_h, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    fclose(fid);
end

if XIdata
    fname = 'xi.txt';
    fid = fopen(fname,'w');
    dlmwrite(fname, ['% ' dirname2], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, xi, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);

    fclose(fid);
    
    fname = 'intData.m';
    fid = fopen(fname,'w');
    dlmwrite(fname, ['% ' dirname2], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'n = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, n_s, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'm1 = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m1_s, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'm2 = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m2_s, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'm3 = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m3_s, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'ks = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, ks, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    if prins_comp   
        dlmwrite(fname, 'rho = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
            dlmwrite(fname, spot, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
        dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    end
    
    dlmwrite(fname, 'piecewise = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, piecewise, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    if piecewise && prins_comp
        dlmwrite(fname, 'r_i = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
            dlmwrite(fname, r_i, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
        dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        
        dlmwrite(fname, 'ksL = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
            dlmwrite(fname, ksL, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 4);
        dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
        ppmat = coef*breaks;
        dlmwrite(fname, 'ppoints_arr = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
            for t = 1:24
                dlmwrite(fname, ppmat(t,:)', 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 5);
            end                
        dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
   
    elseif piecewise && ~prins_comp
        dlmwrite(fname, 'r_i = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
            dlmwrite(fname, r_i, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
        dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
        dlmwrite(fname, 'ppoints_arr = [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
            for i = 1:24
                dlmwrite(fname, ppoints{i}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
            end
        dlmwrite(fname, '];', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    end
    
    fclose(fid);
end


    H = cell(S,1);
    e1 = zeros(k,1);
    e1(1) = 1;
    H_length = 0;
    for s = 1:S
    	H{s} = (W - h*e1')*Mom*P{s}';
        H_length = H_length + size(H{s},1)*size(H{s},2);
    end
if DUALdata
    fname = 'DUALdata.txt';
    fid = fopen(fname,'w');
    dlmwrite(fname, ['! ' dirname2], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    
    %------------ P ----------------------------------
    dlmwrite(fname, 'P_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(P_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1); 
    
    dlmwrite(fname, 'P_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, P{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    %-------------------------------------------------
    
    
    
    dlmwrite(fname, 'T : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, S, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'l : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, l, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'n : [',  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, n_s,  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'kt : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, ks', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'm1 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m1_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'm2 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m2_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'm3 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m3_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A2_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A2_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'H_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(H_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    
    dlmwrite(fname, 'B1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(B1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'B3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(B3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'D_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(D_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A_eq_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A_eq_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);


    dlmwrite(fname, 'D_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, D{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'H_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, H{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    
    dlmwrite(fname, 'A1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            for tau = 1:s
                dlmwrite(fname, A1{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A2_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            for tau = 1:2
                dlmwrite(fname, A2{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, A3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A_eq_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:26
            for tau = 1:s
                dlmwrite(fname, A_eq{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'B1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, B1{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'B3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, B3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'M : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, Mom, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'starts : [',  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, starts,  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    fclose(fid);
    
end

if scenarioData && piecewise
    
    fname = 'scenarioData.txt';
    fid = fopen(fname,'w');
    dlmwrite(fname, ['# ' dirname2], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    dlmwrite(fname, '# constructing scen tree', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ['TYPE    FAN'], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ['TIME    ', num2str(24*days+1)], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ['SCEN    ', num2str(n_scen)], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    dlmwrite(fname, 'RANDOM  1', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    dlmwrite(fname, 'DATA', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    for s = 1:n_scen
        dlmwrite(fname, num2str(1/n_scen), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, '1', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
        dlmwrite(fname, num2str(spot(s,:)'), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    end
    
    dlmwrite(fname, 'END', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    fclose(fid);
    
    PricePoints = cell(24,1);
    for t = 1:24
        PricePoints{t} = zeros(size(ppoints{t} ,1)+2,1);
        PricePoints{t}(1) = rho_low(t);
        PricePoints{t}(2:end-1) = rho_low(t) + ppoints{t};
        PricePoints{t}(end) = rho_low(t) + xi_max(t);
    end
    
    n_Pricepoint = 0;
    for t = 1:24
        n_Pricepoint = n_Pricepoint + size(PricePoints{t},1);
    end
    
    fname = 'PriceData.txt';
    fid = fopen(fname,'w');
    dlmwrite(fname, ['! ' dirname2], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    dlmwrite(fname, ['agg : ', num2str(agg)], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ['Im_addition : ', num2str(imb)], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    dlmwrite(fname, ['Start_h : ', num2str(start_h)], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    dlmwrite(fname, ['n_Pricepoint : ', num2str(n_Pricepoint)], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    
    dlmwrite(fname, 'I : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(r_i+1), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    
    dlmwrite(fname, 'Pricepoint_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    for t = 1:24
        dlmwrite(fname, num2str(PricePoints{t}), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    
    dlmwrite(fname, 'Rho_min : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(rho_low), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    dlmwrite(fname, 'Rho_max : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(rho_low + xi_max), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    
    dlmwrite(fname, 'Inflow_OR : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, infl_out, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1, 'precision', 10);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);  
    
    
    
    
    
    fclose(fid);

end
    
if DualOrg
    % REMOVE START UP VARIABLES:
    for s = 3:S
        C{s}(:,[3:5 9:11]) = [];
        for tau = 3:s
            A1{s,tau}(:,[3:5 9:11]) = [];
        end
        A3{s}(:,[3:5 9:11]) = [];
        n_s(s) = n_s(s) - 6;
    end
    for s = 3:26
        for tau = 3:s
            A_eq{s,tau}(:,[3:5 9:11]) = [];
        end
    end
    
    A_EV = [zeros(1,8) 1 -1];
        
    D_1 = cell(S,1);
    D_3 = cell(S,1);
    D1_length = 0;
    D3_length = 0;
    for s = 1:S
        D_1{s} = P{s}*Mom*P{s}'*B1{s}';
        D1_length = D1_length + size(D_1{s},1)*size(D_1{s},2);
        D_3{s} = P{s}*Mom*P{s}'*B3{s}';
        D3_length = D3_length + size(D_3{s},1)*size(D_3{s},2);
    end

    
    
    
    fname = 'DUALdata_ORG.txt';
    fid = fopen(fname,'w');
    dlmwrite(fname, ['! ' dirname2], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  0);
    
    
    
    
    
    %------------ P ----------------------------------
    dlmwrite(fname, 'P_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(P_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1); 
    
    dlmwrite(fname, 'P_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, P{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    %-------------------------------------------------
    
    
    
    dlmwrite(fname, 'T : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, S, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'l : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, l, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'n : [',  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, n_s,  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'kt : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, ks', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'm1 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m1_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'm3 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m3_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'B1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(B1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'B3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(B3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'D1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(D1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'D3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(D3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A_eq_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A_eq_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);


    dlmwrite(fname, 'D1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, D_1{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'D3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, D_3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            for tau = 1:s
                dlmwrite(fname, A1{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, A3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A_eq_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:26
            for tau = 1:s
                dlmwrite(fname, A_eq{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A_EV : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, A_EV, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'B1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, B1{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'B3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, B3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'M : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, Mom, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'W : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, W, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'h : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, h, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'M_1 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, M_t{1}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'M_2 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, M_t{2}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    fclose(fid);
end
    
if LDRdata && prins_comp
    
    if piecewise
        % Cummulative r_i's:
        r_i_cum = zeros(25,1);
        r_i_cum(1) = 1;
        for t = 1:24
            r_i_cum(t+1) = r_i_cum(t) + r_i(t);
        end
    end
    
    R_AF = [1;zeros(24,1)];
    for i = 1:n_pr
        for j = 1:r_i(i)
            R_AF = [R_AF [0;coef(:,i)]];            
        end        
    end
    R_AF = [R_AF zeros(25,1)];
    
    
    R_mat = [1;zeros(sum(r_i),1)];
    for i = 1:n_pr
        adder = [zeros(1,r_i(i))];
        for j = 1:24
            adder = [adder; coef(j,i)*eye(r_i(j))];
        end
        R_mat = [R_mat adder];
    end
    
    R_mat2 = [R_mat zeros(1+sum(r_i),k-ks(2))];
    D = cell(S,1);
    D_length = 0;
    D{2} = R_mat2*Mom*R_mat2'*C{2}';        % Principal components only first day 
    D_length = D_length + size(D{2},1)*size(D{2},2);
    for s = 3:26
        D{s} = P{s}*Mom*R_AF'*C{s}';        % Principal components only first day 
        D_length = D_length + size(D{s},1)*size(D{s},2);
    end
    for s = 27:S
        D{s} = P{s}*Mom*P{s}'*C{s}';
        D_length = D_length + size(D{s},1)*size(D{s},2);
    end
    
    
    
    fname = 'LDRdata_PCR.txt';
    fid = fopen(fname,'w');
    dlmwrite(fname, ['! ' dirname2], 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'T : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, S, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'l : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, l, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'n : [',  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, n_s,  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'n_pr : [',  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, n_pr,  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'starts : [',  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, starts,  'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'kt : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, ks', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'kt_OR : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, ksL', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'm1 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m1_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'm2 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m2_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'm3 : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, m3_s, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'piecewise : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, piecewise, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    if piecewise
    dlmwrite(fname, 'r_i_cum : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, r_i_cum, 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    else
        dlmwrite(fname, 'r_i_cum : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
            dlmwrite(fname, zeros(25,1), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    end
    dlmwrite(fname, 'R : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, R_mat, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A2_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A2_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    
    dlmwrite(fname, 'B1_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(B1_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'B3_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(B3_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'D_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(D_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A_eq_length : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, num2str(A_eq_length), 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);


    dlmwrite(fname, 'D_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, D{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            for tau = 1:s
                dlmwrite(fname, A1{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'A2_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            for tau = 1:2
                dlmwrite(fname, A2{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, A3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'A_eq_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:26
            for tau = 1:s
                dlmwrite(fname, A_eq{s,tau}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
            end
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'W : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, W, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'B1_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, B1{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'B3_arr : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        for s = 1:S
            dlmwrite(fname, B3{s}, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
        end
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
    
    dlmwrite(fname, 'h : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, h, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    dlmwrite(fname, 'nu : [', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);
        dlmwrite(fname, nu, 'delimiter', ' ', 'newline', 'pc', '-append', 'roffset',  1);
    dlmwrite(fname, ']', 'delimiter', '', 'newline', 'pc', '-append', 'roffset',  1);

    fclose(fid);
end

    

disp('Data files written, new folder created');
